/*
 * Copyright 2008 Evan Cowden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.nakedosgi.processor;

import java.util.Map;
import java.util.Set;
import javax.annotation.processing.AbstractProcessor;
import javax.annotation.processing.ProcessingEnvironment;
import javax.annotation.processing.RoundEnvironment;
import javax.lang.model.element.TypeElement;

/**
 * An annotation processor specifically for annotations that generate
 * configuration for an OSGi bundle
 * @author Evan Cowden
 */
public abstract class OSGiProcessor extends AbstractProcessor {

    /**
     * The bundle's manifest
     */
    protected Map<String, String> manifest;

    @Override
    public synchronized void init(ProcessingEnvironment processingEnv) {
        super.init(processingEnv);

        OSGiProcessorManager manager = OSGiProcessorManager.getInstance();

        //Get a local reference to the manifest
        manifest = manager.getManifest();
    }

    @Override
    /**
     * DO NOT TRY TO OVERRIDE THIS METHOD.  Instead, implement  the 
     * {@link OSGiProcessor.processBundle()} method.
     */
    public final boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {
        //Register this processor as active with the manager
        OSGiProcessorManager manager = OSGiProcessorManager.getInstance();
        manager.registerBundleProcessor(this, processingEnv);
        
        //Delegate processing to the child
        boolean result = processBundle(annotations, roundEnv);

        //Notify the manager if processing is complete
        if (roundEnv.processingOver()) {
            manager.processorFinished(this);
        }

        return result;
    }

    /**
     * <p>Implement this method instead of {@code process()}</p>
     * 
     * <p>Processes a set of annotation types on type elements
     * originating from the prior round and returns whether or not
     * these annotations are claimed by this processor.  If {@code
     * true} is returned, the annotations are claimed and subsequent
     * processors will not be asked to process them; if {@code false}
     * is returned, the annotations are unclaimed and subsequent
     * processors may be asked to process them.  A processor may
     * always return the same boolean value or may vary the result
     * based on chosen criteria.</p>
     *
     * <p>The input set will be empty if the processor supports {@code
     * "*"} and the root elements have no annotations.  A {@code
     * Processor} must gracefully handle an empty set of annotations.</p>
     *
     * @param annotations the annotation types requested to be processed
     * @param roundEnv  environment for information about the current and prior round
     * @return whether or not the set of annotations are claimed by this processor
     */
    public abstract boolean processBundle(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv);

    /**
     * Called when processing is completed.  Implement this method to have
     * specific functionality executed after all processing is done.
     */
    public void processingDone() {}
}
