/*
 * Copyright 2008 Evan Cowden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.nakedosgi.processor;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;
import java.util.Set;
import java.util.SortedSet;
import java.util.TreeSet;
import javax.lang.model.element.Element;
import org.nakedosgi.manifest.ManifestConstants;

/**
 *
 * @author Evan Cowden
 */
public class OSGiProcessors {

    /**
     * Get seperate values out of a single manifest attribute, assuming that 
     * the original attribute value is a comma-seperated list of multiple values.
     * @param manifest
     * @param key the name of the target attribute
     */
    protected static Collection<String> getValues(Map<String, String> manifest, String key) {
        String rawValues = manifest.get(key);

        if (rawValues == null) {
            return Collections.EMPTY_LIST;
        }

        String[] values = rawValues.split("[,]");
        Collection<String> result = new ArrayList(values.length);

        for (String value : values) {
            result.add(value.trim());
        }

        return result;
    }
    
    /**
     * Append the specified entries into the manifest without overriding 
     * existing entries
     * @param manifest
     * @param manifestKey the key for the manifest attribute
     * @param entries the entries to append to the key
     */
    protected static void append(Map<String, String> manifest, String manifestKey, Collection<String> entries) {
        entries.addAll(getValues(manifest, manifestKey));

        //Build string from list
        StringBuilder builder = new StringBuilder();
        for (String entry : entries) {
            builder.append(entry);
            builder.append(ManifestConstants.MANIFEST_SEPERATOR);
        }

        //...remove last ", "
        int index = builder.lastIndexOf(ManifestConstants.MANIFEST_SEPERATOR);
        if (index != -1) {
            builder.delete(index, index + ManifestConstants.MANIFEST_SEPERATOR.length());
        }

        manifest.put(manifestKey, builder.toString());
    }
    
    /**
     * Append the specified elements into the manifest without overriding 
     * existing entries
     * @param elements the elements to place in the manifest (as transformed by 
     * {@code .asType().toString()}
     * @param manifest
     * @param manifestKey
     */
    protected static void append(Map<String, String> manifest, String manifestKey, Set<? extends Element> elements) {
        SortedSet<String> entries = new TreeSet();

        for (Element element : elements) {
            entries.add(element.asType().toString());
        }
        append(manifest, manifestKey, entries);
    }
}
