/*
 * Copyright 2008 Evan Cowden
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License. 
 * You may obtain a copy of the License at 
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0 
 * 
 * Unless required by applicable law or agreed to in writing, software 
 * distributed under the License is distributed on an "AS IS" BASIS, 
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. 
 * See the License for the specific language governing permissions and 
 * limitations under the License.
 */
package org.nakedosgi.processor.scr;

import java.util.Map;
import java.util.Set;
import javax.annotation.processing.Processor;
import javax.annotation.processing.RoundEnvironment;
import javax.annotation.processing.SupportedAnnotationTypes;
import javax.annotation.processing.SupportedSourceVersion;
import javax.lang.model.SourceVersion;
import javax.lang.model.element.Element;
import javax.lang.model.element.ExecutableElement;
import javax.lang.model.element.TypeElement;
import javax.lang.model.element.VariableElement;
import javax.lang.model.type.MirroredTypeException;
import org.nakedosgi.Constants;
import org.nakedosgi.annotations.NoValue;
import org.nakedosgi.annotations.scr.SCRBind;
import org.nakedosgi.processor.OSGiProcessor;
import org.nakedosgi.processor.scr.model.Component;
import org.nakedosgi.processor.scr.model.Reference;
import org.nakedosgi.spi.ServiceProvider;

/**
 * Processor for the {@link SCRBind} annotation
 * @author Evan Cowden
 */
@ServiceProvider(Processor.class)
@SupportedSourceVersion(SourceVersion.RELEASE_6)
@SupportedAnnotationTypes("org.nakedosgi.annotations.scr.SCRBind")
public class SCRBindProcessor extends OSGiProcessor {

    @Override
    public boolean processBundle(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {
        Set<? extends Element> elements = roundEnv.getElementsAnnotatedWith(SCRBind.class);
        Map<String, Component> components = SCRProcessingUtil.getComponents();
        
        for (Element rawElement : elements) {
            ExecutableElement element = (ExecutableElement) rawElement;

            //Verify that the method has exactly one parameter
            if (BindUnbindUtil.parametersAreInvalid(element)) {
                continue;
            }

            //Get all necessary objects
            //...annotation & name
            SCRBind annotation = element.getAnnotation(SCRBind.class);
            String annotationName = annotation.name();
            String name = BindUnbindUtil.getName(element, annotationName);

            //...Model components
            Component component = SCRProcessingUtil.getComponent(element, components);
            Reference reference = BindUnbindUtil.getReference(name, component);

            //Copy info over from annotation
            //...cardinality
            reference.setCardinality(annotation.cardinality());

            //...interface
            //(This is where I start thinking that the person that designed this API needs to be shot...)
            String declaredClassName = null;
            try {
                annotation.interface_().getName(); //Will throw MirroredTypeException
            } catch (MirroredTypeException ex) {
                declaredClassName = ex.getTypeMirror().toString();
            }
            if (declaredClassName.equals(NoValue.class.getName())) {
                VariableElement parameter = element.getParameters().get(0);
                reference.setInterface(parameter.asType().toString());
            } else {
                reference.setInterface(declaredClassName);
            }

            //...policy
            reference.setPolicy(annotation.policy());

            //...target
            if (!annotation.target().equals(Constants.NO_VALUE)) {
                reference.setTarget(annotation.target());
            }

            //...bind
            String bind = element.getSimpleName().toString();
            reference.setBind(bind);
        }
        
        return true;
    }
}
